{-----------------------------------------------------------------------------
 Unit Name: uFormularManager
 Copyright: german medicalcoders
 Author:    Erhard Kieling
 Date:      17-Aug-2007

 License:   LGPL
 *****************************************************************************
 *                                                                           *
 *  See the at:                                                              *
 *    http://www.gnu.org/licenses/lgpl.html                                  *
 *  for details about the copyright.                                         *
 *                                                                           *
 *  This program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *                                                                           *
 *****************************************************************************

 Purpose:
   This Component could be used for a formular-management with multiuser
   methods

 History:
-----------------------------------------------------------------------------}
unit uFormularManager;

interface

uses
  SysUtils, Classes, Messages,
  {$IFDEF FPC}
    Forms;
  {$ELSE}
    {$IFDEF MSWINDOWS}
      Forms;
    {$ENDIF}
    {$IFDEF LINUX}
      QForms;
    {$ENDIF}
  {$ENDIF}

const
  cModulName = 'uFormularManager';

type

  TUserDescriptor = class(TObject)
    Username   : string;
    LoginAt    : TDateTime;
    deactivated: TDateTime;
  end; // of TUserDescriptor

  TFormDescriptor = class(TObject)
    FormOwner: TComponent;
    FormClass: TFormClass;
    FormObj  : TObject;
    FormName : string[60];
    UserName : string[32];
    OpenAt   : TDateTime;
    FormInfo : string;
  end; // of TFormDescriptor

  {---------------------------------------------------------------------------
    Copyright: german medicalcoders
    Author:    Erhard Kieling
    Date:      05-Mai-2006
    Unit:      FormularManager
    Subject:
    Event procdures and functions
  ---------------------------------------------------------------------------}

  TOnReleaseFormEvent   = procedure(Sender: TObject) of object;
  TOnFormCloseSelfEvent = procedure(Sender: TObject) of object;
  TOnAddNewFormEvent    = function(AFormDesc: TFormDescriptor;
                                   AFormInfo: string): TObject of object;
  TOnLogFormInfoEvent   = procedure(Sender: TObject;
                                    AFormDesc: TFormDescriptor;
                                    Action: string) of object;
  TOnLogUserEvent       = procedure(Sender: TObject;
                                    AUserDesc: TUserDescriptor;
                                    Action: string) of object;

  {---------------------------------------------------------------------------
    Copyright: german medicalcoders
    Author:    Erhard Kieling
    Date:      16-Mai-2006
    Unit:      FormularManager
    Subject:   Objekt-Deklaration
    Kontrolliert Formulare und schliet diese bei Bedarf oder automatisch
    bei Programm-Ende
  ---------------------------------------------------------------------------}
  TFormularManager = class(TComponent)
  private
    { Private-Deklarationen }
    FFormList: TList;
    FUserList: TList;
    FFormShowAuto: boolean;
    FActiveUser: TUserDescriptor;
    FOnLogFormInfo: TOnLogFormInfoEvent;
    FOnLogUser: TOnLogUserEvent;
  protected
    { Protected-Deklarationen }

    function GetFormShowAuto: boolean;
    procedure SetFormShowAuto(Value: boolean);

    function GetShowMethod(AFormClass: TFormClass; AObj: TObject): pointer;
    function GetShowModalMethod(AFormClass: TFormClass; AObj: TObject): pointer;
    function LookForThisForm(Sender: TObject; var i: integer): boolean;
    procedure SetActiveUser(Value: TUserDescriptor);
  public
    { Public-Deklarationen }

    constructor Create(AOwner: TComponent); override;
    destructor  Destroy;

    function  DoAddNewForm(AFormDesc: TFormDescriptor;
                           AFormInfo: string): TObject;
    function  AddNewUser(AUserDesc: TUserDescriptor): boolean;
    procedure DeleteUser(AUserDesc: TUserDescriptor);
    function  LookForThisUser(AUserName: string): TUserDescriptor;
    procedure DoReleaseForm(Sender: TObject); overload;
    procedure DoReleaseForm(AFormDesc: TFormDescriptor); overload;
    procedure DoFormCloseSelf(Sender: TObject);
    procedure DoEraseForms(Sender: TObject);
    procedure DoHideForms(Sender: TObject);
    procedure DoUnhideForms(Sender: TObject);

    property FormularList: TList read FFormList;
    property ActiveUser: TUserDescriptor read FActiveUser write SetActiveUser;
  published
    { Published-Deklarationen }

    property FormShowAuto: boolean read GetFormShowAuto write SetFormShowAuto;
    property OnLogFormInfo: TOnLogFormInfoEvent
        read FOnLogFormInfo write FOnLogFormInfo;
    property OnLogUser: TOnLogUserEvent
        read FOnLogUser write FOnLogUser;
  end; // of TFormularManager

var
  Formmanager: TFormularManager;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Application-Management', [TFormularManager]);
end;

{-----------------------------------------------------------------------------
  Procedure: TFormularManager.Create
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      05-Mai-2006
-----------------------------------------------------------------------------}
constructor TFormularManager.Create(AOwner: TComponent);
begin
  { constructor body }
  inherited Create(AOwner);
  FFormList:= TList.Create;
  FUserList:= TList.Create;
  FFormShowAuto:= false;
end; // of TFormularManager.Create

destructor TFormularManager.Destroy;
begin
  DoEraseForms(Self);
  FFormList.Free;
  FUserList.Free;
end; // of TFormularManager.Destroy

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Method:    GetFormShowAuto
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      07-Mai-2006
  Arguments: None
  Result:    boolean
-----------------------------------------------------------------------------}
function TFormularManager.GetFormShowAuto: boolean;
const
  cProcName = 'GetFormShowAuto';
begin
  { function body }
  Result:= FFormShowAuto;
end; // of TFormularManager.GetFormShowAuto

{-----------------------------------------------------------------------------
  Object:    TFormularManager
  Procedure: SetFormShowAuto
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      07-Mai-2006
  Arguments: Value: boolean
-----------------------------------------------------------------------------}
procedure TFormularManager.SetFormShowAuto(Value: boolean);
const
  cProcName = 'SetFormShowAuto';
begin
  { procedure body }
  if (Value <> FFormShowAuto) then FFormShowAuto:= Value;
end; // of TFormularManager.SetFormShowAuto

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Method:    GetShowMethod
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      05-Mai-2006
  Arguments: AFormClass: TFormClass; AObj: TObject
  Result:    pointer
-----------------------------------------------------------------------------}
function TFormularManager.GetShowMethod(AFormClass: TFormClass;
                                        AObj: TObject): pointer;
const
  cProcName = 'GetShowMethod';
begin
  { function body }
  Result:= TForm(AObj).MethodAddress('Show');
end; // of TFormularManager.GetShowMethod

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Method:    GetShowModalMethod
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      05-Mai-2006
  Arguments: AFormClass: TFormClass; AObj: TObject
  Result:    pointer
-----------------------------------------------------------------------------}
function TFormularManager.GetShowModalMethod(AFormClass: TFormClass;
                                             AObj: TObject): pointer;
const
  cProcName = 'GetShowModalMethod';
begin
  { function body }
  Result:= TForm(AObj).MethodAddress('ShowModal');
end; // of TFormularManager.GetShowModalMethod

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Method:    LookForThisForm
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      05-Mai-2006
  Arguments: Sender: TObject; var i: integer
  Result:    boolean
-----------------------------------------------------------------------------}
function TFormularManager.LookForThisForm(Sender: TObject;
                                          var i: integer): boolean;
const
  cProcName = 'LookForThisForm';
var
  good,
  found: boolean;
begin
  { function body }
  Result:= false;
  found:= false; i:= 0;
  good:= (Sender <> nil) and (FFormList.Count > 0);
  if good then while ((i < FFormList.Count) and not(found)) do begin
    try
      found:= (TFormDescriptor(FFormList[i]).FormObj = Sender);
      if not(found) then inc(i);
    except
      raise;
    end;
  end; // if good then while ((i < FFormList.Count) and not(found)) do begin
  Result:= found;
end; // of TFormularManager.LookForThisForm

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Methode:   SetActiveUser
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      17-Aug-2007
  Arguments: Value: TUserDescriptor
-----------------------------------------------------------------------------}
procedure TFormularManager.SetActiveUser(Value: TUserDescriptor);
const
  cProcName = 'uFormularManager.TFormularManager.SetActiveUser';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'Value: TUserDescriptor'
  end; // of function SetErrorParams: string

var
  UserChanged: boolean;
begin
  { procedure body }
  UserChanged:= AnsiCompareStr(Value.Username, ActiveUser.Username) = 0;
  if UserChanged then begin
    DoHideForms(Self);
    if Assigned(FOnLogUser)
      then OnLogUser(Self, FActiveUser, 'deactivate');
    FActiveUser.deactivated:= Now;
    FActiveUser:= Value;
    if Assigned(FOnLogUser)
      then OnLogUser(Self, FActiveUser, 'activate');
    DoUnhideForms(Self);
  end; // of if UserChanged then begin
end; // of TFormularManager.SetActiveUser

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Method:    DoAddNewForm
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      05-Mai-2006
  Arguments: AFormDesc: TFormDescriptor; AFormInfo: string
  Result:    TObject
-----------------------------------------------------------------------------}
function TFormularManager.DoAddNewForm(AFormDesc: TFormDescriptor;
                                       AFormInfo: string): TObject;
const
  cProcName = 'DoAddNewForm';
var
  pShowMethod,
  pShowModal : pointer;
  info       : string;
begin
  { function body }
  Result:= nil;
  if (AFormDesc <> nil) then with AFormDesc do begin
    try { create a new formular }
      FormObj:= FormClass.Create(FormOwner);
      FormName:= TComponent(FormObj).Name;
      FFormList.Add(AFormDesc);
      Result:= AFormDesc.FormObj;
      UserName:= ActiveUser.Username;
      OpenAt:= Now;
      if Assigned(FOnLogFormInfo)
          then OnLogFormInfo(Self, AFormDesc, 'addform');
      AFormDesc.FormInfo:= info + #13#10 + '  [Info: ' + AFormInfo + ']';
      if FormShowAuto then try
        pShowMethod:= GetShowMethod(FormClass, FormObj);
        if (pShowMethod <> nil) then TForm(FormObj).Show;
        if Assigned(FOnLogFormInfo)
            then OnLogFormInfo(Self, AFormDesc, 'showself');
      except
        raise;
      end; // of if FormShowAuto then try
    except
      raise;
    end; // of try { create a new formular }
  end; // of if (AFormDesc <> nil) then with AFormDesc do begin
end; // of TFormularManager.DoAddNewForm

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Methode:   AddNewUser
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      17-Aug-2007
  Arguments: AUserDesc: TUserDescriptor
  Result:    boolean
-----------------------------------------------------------------------------}
function TFormularManager.AddNewUser(AUserDesc: TUserDescriptor): boolean;
const
  cProcName = 'uFormularManager.TFormularManager.AddNewUser';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'AUserDesc: TUserDescriptor'
  end; // of function SetErrorParams: string

begin
  { procedure body }
  Result:= false;
  if (LookForThisUser(AUserDesc.Username) = nil) then begin
    if Assigned(FOnLogUser)
      then OnLogUser(Self, AUserDesc, 'adduser');
    FUserList.Add(AUserDesc);
    Result:= true;
  end;
end; // of TFormularManager.AddNewUser

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Methode:   DeleteUser
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      17-Aug-2007
  Arguments: AUserDesc: TUserDescriptor
  Subject:
    Delete User and release all Formulars for this User
-----------------------------------------------------------------------------}
procedure TFormularManager.DeleteUser(AUserDesc: TUserDescriptor);
const
  cProcName = 'uFormularManager.TFormularManager.DeleteUser';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'AUserDesc: TUserDescriptor'
  end; // of function SetErrorParams: string

  function UserOK(S1, S2: string): boolean;
  begin
    Result:= AnsiCompareStr(S1, S2) = 0;
  end;

var
  i       : integer;
  found   : boolean;
begin
  { procedure body }
  if (FUserList.IndexOf(AUserDesc) >= 0) then begin
    i:= 0; found:= false;
    while (i <= FFormList.Count - 1) and not(found) do begin
      found:= UserOk(TFormDescriptor(FFormList.Items[i]).UserName,
                     AUserDesc.Username);
      if found then begin
        DoReleaseForm(TFormDescriptor(FFormList.Items[i]));
        found:= false;
        i:= 0;
      end // of if found then begin
      else inc(i);
    end; // of while (i <= FFormList.Count - 1) do begin
    if Assigned(FOnLogUser)
      then OnLogUser(Self, AUserDesc, 'deleteuser');
  end; // of if (FUserList.IndexOf(AUserDesc) >= 0) then begin
end; // of TFormularManager.DeleteUser

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Method:    LookForThisUser
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      17-Aug-2007
  Arguments: AUserName: string
  Result:    TUserDescriptor
-----------------------------------------------------------------------------}
function TFormularManager.LookForThisUser(AUserName: string): TUserDescriptor;
const
  cProcName = 'uFormularManager.TFormularManager.LookForThisUser';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'AUserName: string'
    // 'TUserDescriptor';
  end; // of function SetErrorParams: string

  function UserOK(S1, S2: string): boolean;
  begin
    Result:= AnsiCompareStr(S1, S2) = 0;
  end; // of UserOK

var
  i    : integer;
  found: boolean;
begin
  { function body }
  Result:= nil;
  if (AUserName <> '') then begin
    i:= 0; found:= false;
    while (i <= FUserList.Count - 1) and not(found) do begin
      found:= UserOK(TUserDescriptor(FUserList[i]).Username, AUserName);
      if found
        then Result:= TUserDescriptor(FUserList[i])
        else inc(i);
    end; // of while (i <= FUserList.Count - 1) and not(found) then begin
  end; // of if (AUserName <> '') then begin
end; // of TFormularManager.LookForThisUser

{-----------------------------------------------------------------------------
  Object:    TFormularManager
  Procedure: DoReleaseForm
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      05-Mai-2006
  Arguments: Sender: TObject
-----------------------------------------------------------------------------}
procedure TFormularManager.DoReleaseForm(Sender: TObject);
const
  cProcName = 'DoReleaseForm';
var
  AFormDesc: TFormDescriptor;
  i        : integer;
begin
  { procedure body }
  if LookForThisForm(Sender, i) then begin
    AFormDesc:= TFormDescriptor(FFormList[i]);
    if (AFormDesc.FormObj <> nil) then try
      if Assigned(FOnLogFormInfo)
        then OnLogFormInfo(Self, AFormDesc, 'release');
      AFormDesc.FormObj.Free;
    except
      AFormDesc.FormObj:= nil;
    end; // of if (AFormDesc.FormObj <> nil) then try
    FFormList.Delete(FFormList.IndexOf(Pointer(AFormDesc)));
    AFormDesc:= nil;
    FFormList.Capacity:= FFormList.Count;
  end; // of if LookForThisForm(Sender, i) then begin
end; // of TFormularManager.DoReleaseForm

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Methode:   DoReleaseForm
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      17-Aug-2007
  Arguments: AFormDesc: TFormDescriptor
-----------------------------------------------------------------------------}
procedure TFormularManager.DoReleaseForm(AFormDesc: TFormDescriptor);
const
  cProcName = 'uFormularManager.TFormularManager.DoReleaseForm';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'AFormDesc: TFormDescriptor'
  end; // of function SetErrorParams: string

begin
  { procedure body }
  if (AFormDesc.FormObj <> nil) then try
    if Assigned(FOnLogFormInfo)
      then OnLogFormInfo(Self, AFormDesc, 'release');
    AFormDesc.FormObj.Free;
  except
    AFormDesc.FormObj:= nil;
  end; // of if (AFormDesc.FormObj <> nil) then try
  FFormList.Delete(FFormList.IndexOf(Pointer(AFormDesc)));
  AFormDesc:= nil;
  FFormList.Capacity:= FFormList.Count;
end; // of TFormularManager.DoReleaseForm

{-----------------------------------------------------------------------------
  Object:    TFormularManager
  Procedure: DoFormCloseSelf
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      05-Mai-2006
  Arguments: Sender: TObject
-----------------------------------------------------------------------------}
procedure TFormularManager.DoFormCloseSelf(Sender: TObject);
const
  cProcName = 'DoFormCloseSelf';
var
  AFormDesc: TFormDescriptor;
  i        : integer;
begin
  { procedure body }
  if LookForThisForm(Sender, i) then begin
    AFormDesc:= TFormDescriptor(FFormList[i]);
    if Assigned(FOnLogFormInfo)
      then OnLogFormInfo(Self, AFormDesc, 'closeself');
    FFormList.Delete(i);
    AFormDesc:= nil;
    FFormList.Capacity:= FFormList.Count;
  end; // of if LookForThisForm(Sender, i) then begin
end; // of TFormularManager.DoFormCloseSelf

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Methode:   DoEraseForms
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      17-Mai-2006
  Arguments: Sender: TObject
-----------------------------------------------------------------------------}
procedure TFormularManager.DoEraseForms(Sender: TObject);
const
  cProcName = 'DoEraseForms';
var
  AFormDesc: TFormDescriptor;
begin
  { procedure body }
  while (FFormList.Count > 0) do begin
    try
      AFormDesc:= TFormDescriptor(FFormList[FFormList.Count - 1]);
      if Assigned(FOnLogFormInfo)
        then OnLogFormInfo(Self, AFormDesc, 'erase');
      AFormDesc.FormObj.Free;
      FFormList.Delete(FFormList.Count - 1);
    except
      raise;
    end;
  end; // of while (FFormList.Count > 0) do begin
end; // of TFormularManager.DoEraseForms

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Methode:   DoHideForms
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      22-Jun-2006
  Arguments: Sender: TObject
-----------------------------------------------------------------------------}
procedure TFormularManager.DoHideForms(Sender: TObject);
const
  cProcName = 'uFormularManager.TFormularManager.DoHideForms';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'Sender: TObject'
  end; // of function SetErrorParams: string

  function UserOK(S1, S2: string): boolean;
  begin
    Result:= AnsiCompareStr(S1, S2) = 0;
  end; // of function UserOK(S1, S2): boolean

var
  AFormDesc: TFormDescriptor;
  index    : integer;
begin
  { procedure body }
  index:= 0;
  while (index < FFormList.Count) do begin
    try
      AFormDesc:= TFormDescriptor(FFormList[index]);
      if UserOK(AFormDesc.UserName, ActiveUser.Username) then begin
        if Assigned(FOnLogFormInfo)
          then OnLogFormInfo(Self, AFormDesc, 'hide');
        TForm(AFormDesc.FormObj).Hide;
      end;
      inc(index);
    except
      raise;
    end;
  end; // of while (FFormList.Count > 0) do begin
end; // of TFormularManager.DoHideForms

{-----------------------------------------------------------------------------
  Class:     TFormularManager
  Methode:   DoUnhideForms
  Copyright: german medicalcoders
  Author:    Erhard Kieling
  Date:      23-Jun-2006
  Arguments: Sender: TObject
-----------------------------------------------------------------------------}
procedure TFormularManager.DoUnhideForms(Sender: TObject);
const
  cProcName = 'uFormularManager.TFormularManager.DoUnhideForms';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'Sender: TObject'
  end; // of function SetErrorParams: string

  function UserOK(S1, S2: string): boolean;
  begin
    Result:= AnsiCompareStr(S1, S2) = 0;
  end; // of function UserOK(S1, S2): boolean

var
  AFormDesc: TFormDescriptor;
  index    : integer;
begin
  { procedure body }
  index:= 0;
  while (index < FFormList.Count) do begin
    try
      AFormDesc:= TFormDescriptor(FFormList[index]);
      if  UserOK(AFormDesc.UserName,ActiveUser.Username) and
          not(TForm(AFormDesc.FormObj).Visible)
      then begin
        if Assigned(FOnLogFormInfo)
          then OnLogFormInfo(Self, AFormDesc, 'unhide');
        TForm(AFormDesc.FormObj).Visible:= true;
      end;
      inc(index);
    except
      raise;
    end;
  end; // of while (FFormList.Count > 0) do begin
end; // of TFormularManager.DoUnhideForms

end.


